/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2000, 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**
The networkExec class should be sub-classed by the modeler to create
network executives for DSDEVS coupled models.
*/

class networkExec: public atomic {

public:		
/// Creates a network executive with an empty name string.
		networkExec ();
/// Creates a network executive and assigns it the indicated name.
		networkExec (const char* name);
/**
The indexed returned contains EventReceivers.  The recievers 
are assumed to be owned by the model, and so are not deleted by the
simulator. 
*/
		virtual const indexed* computeNeighbors (devs& model,
			const port_value& pv) = 0;

/// Returns the component set for the networkExec.
		const set* getComponents () const;
/// This deletes all of the models in the component set.
		virtual ~networkExec ();

/// This method supports the simulator.
		virtual void _reset (stime_t t = 0.0);
/// This method supports the simulator.
		void makeStructureChanges (set& new_models, set& del_models);
/// This method supports the simulator.
		void set_network(devs* network) {
		   this->network = network;
		   } 
protected:
/// Add a devs to the set of components for this network executive.
		void add (devs* model);
/**
Remove a devs from the set of components for this network executive.  The model
is deleted at the end of the next simulation cycle.  If the model is undefined,
this method simply returns.
*/
		void remove (devs* model);
/**
Returns the network model for which this model is the network executive.
The returned value is NULL until the reset method of the network executive
has been called.  The network model should not be deleted.
*/
		devs* getNetwork() const { return network; }

private:
		devs* network;
		set components; 
		set new_models;
		set del_models;

};

