/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000, 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**
The devsn class is the base class for representing a DEVS network models.
*/

class devsn: public devs {

public:		
/// Creates a devs network model without any component models.
		devsn ():devs (devs::NETWORK) { }
		devsn (const char* name):devs (name, devs::NETWORK) { }
/**
The indexed returned contains EventReceivers.  The recievers 
are assumed to be owned by the model, and so are not deleted by the
simulator. 
*/
		virtual const indexed* computeNeighbors (devs& model,
			const port_value& pv) = 0;
/// Returns a set holding the components of this coupled model.
		virtual const set* getComponents () const = 0;
/**
This method is called after the network model has undergone a state change.
The two sets that are supplied should be filled with models to be added
and deleted from the network model.  Models in the add set are owned by
the network model.  Models in the delete set are removed by the simulator.
*/
		virtual void makeStructureChanges (set& add_models, set& del_models) = 0;
/**
Resets the model and all of its components.  This method is used to
support the simulator.
*/
		virtual void _reset (stime_t t0 = 0.0);
/// Deletes the model and all of its components.
		virtual ~devsn ();

};

