/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**
Wrapper class around an OS native mutex.  This implementation assumes
a POSIX pthreads compliant interface to the native thread library.
*/
class Mutex {

public:
/// Create an unlocked mutex
		inline Mutex () { 
			#ifdef PARALLEL_POSIX
			pthread_mutex_init (&mtx, NULL); 
			#endif
			}
/// Block until the mutex is available, then grab it
		inline void lock () { 
			#ifdef PARALLEL_POSIX
			pthread_mutex_lock (&mtx); 
			#endif
			}
/// Release the mutex
		inline void unlock () { 
			#ifdef PARALLEL_POSIX
			pthread_mutex_unlock (&mtx); 
			#endif
			}
/// Delete the mutex
		inline ~Mutex () { 
			#ifdef PARALLEL_POSIX
			pthread_mutex_destroy (&mtx); 
			#endif
			}

private:
		#ifdef PARALLEL_POSIX
		pthread_mutex_t mtx;
		#endif

};
