/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**
Parallel simulator for devsn models.  This class is used internally
to do thread based parallel simulation.  This class overrides
each method of the DEVScoordinator to make it threadsafe.  Also
includes threaded functions to compute state changes and I/O in
a threadsafe way.
*/
class pDEVScoordinator: private psimulator, public DEVScoordinator {

public:		
		pDEVScoordinator ();
// Methods from DEVScoordinator
		virtual void computeNextOutput (stime_t t);
		virtual void computeIO (stime_t t);
		virtual void computeTransition (stime_t t);
// Join with parallel threads
		virtual void wait ();
// Destructor
		virtual ~pDEVScoordinator ();

private:
// Mutex used to prevent concurrent access to the simulator
		Mutex mtx;
// Thread for doing parallel processing of I/O and state change computations
		Thread* thr;
// Variable for passing time arguments to the parallel functions
		stime_t t_thread;
// Functions for doing parallel computation
		friend void coord_compute_next_output (void* arg);
		friend void coord_compute_io (void* arg);
		friend void coord_compute_transition (void* arg);
		void _computeNextOutput (stime_t t);
		void _computeIO (stime_t t);
		void _computeTransition (stime_t t);

};

