/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000, 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/// Heap priority type
typedef double priority_t;
 
/**
A priority queue, implemented as a binary heap.
*/
class pqueue: public object {
	public:		
		/// Minimum allowable priority
		static const priority_t min_priority;

		/// Creates a heap with initial capacity and incremental growth value.
		pqueue();
		pqueue(unsigned capacity, unsigned grow = DEFAULT_CONTAINER_GROWTH); 
		/**
		Inserts a new item into the queue.  Inserting an object twice has
		undefined effects.
		*/
		void insert(object *obj, priority_t priority);
		/// Removes the element at the front of the queue.
		object* removeMinimum(); 
		/**
		Repositions an item already in the queue.  Undefined if an
		item is not in the queue.
		*/
		void adjust(const object &obj, priority_t new_priority);
		/**
		Returns the priority of an enqueued item.  Undefined item item is not
		in the queue.
		*/
		priority_t getPriority(const object &obj) const;
		/// Returns true if the item has been enqueued, false otherwise.
		bool isEnqueued(const object &obj) const;
		/**
		Returns the minimum priority.  Returns min_priority if the
		queue is empty.
		*/
		priority_t minPriority() const;
		/// Returns the number of items in the queue.
		inline unsigned getSize() const { return members->keyCount(); }
		/// Destructor.  Does not destroy queued objects.
		~pqueue();
	private:
		// Defn of an element in the heap
		class _element {
			public:
				object *item;
				priority_t priority;
				inline _element() {}
				inline _element(object* obj, priority_t key) {
					item = obj;
					priority = key;
				}
		};

		hashtable* members;
		_element ** heap;
		unsigned capacity;
		unsigned grow;
		
		void enlarge();
		// do not call with index = 0
		unsigned percolate_up(unsigned index, priority_t priority);
		unsigned percolate_down(unsigned index, priority_t priority);
};
