/***************
mtrand, a C++ wrapper around the Mersenne Twister psuedo random
number generator provided by M. Matsumoto and T. Nishimura
Copyright (C) 2000, 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/

/**
The mtrand class is a random number generator based on a GNU implementation
of the Mersenne Twister (see mtrand.cpp for more info).  Each copy
of the object has its own state, and so multiple copies
will produce independent random number streams.
*/

class mtrand: public random_seq {

public:
/// Create a generator with an unsigned long seed 
		mtrand (unsigned long seed = 1);
/// Create a generator with an arbitrary length seed
		mtrand (const unsigned long* seed_array); 
/// Copy constructor performs a deep copy
		mtrand (const mtrand& src);
/// Assignment operator does a deep copy
		const mtrand& operator= (const mtrand& src);
/// Set the seed
		void set_seed (unsigned long seed);
/// Create a copy of this mtrand
		random_seq* copy () const;
/// Get the next unsigned long value.
		unsigned long next_long ();
/// Get the next value in the range [0,1]
		double next_dbl ();
/// Destructor
		~mtrand ();

private:	
// Wrappers around the original function calls
		void sgenrand (unsigned long);
		void lsgenrand (const unsigned long*);
		unsigned long genrand ();

// The array for the state vector
		unsigned long* mt;
		int mti;
		unsigned long init_seed;

};

