/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000, 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <adevs/adevs.h>
using namespace adevs;

/**
It is important to avoid calls to virtual DEVSsimulator methods
within the DEVSsimulator object.  This may result in deadlocks
during parallel simulation due to multiple attempts to acquire
the monitor for the parallel object.
*/

DEVSsimulator::DEVSsimulator ():simulator (simulator::SIM) { 

recvd_input = false;
model = NULL;

}

DEVSsimulator::DEVSsimulator (simulator::SimType type):simulator (type) {

recvd_input = false;
model = NULL;

}

void DEVSsimulator::reset (stime_t t0) {

recvd_input = false;

}

void DEVSsimulator::computeNextOutput (stime_t t) {

// only call output function if the time is tN
if (t == model->timeNext ())
   model->output_func ();

}
  
void DEVSsimulator::computeIO (stime_t t) {

// only call output function if the time is tN
if (t == model->timeNext ())
   model->output_func ();

}

void DEVSsimulator::setModel (atomic* model) {

simulator::setModel (model);
this->model = model;

}

void DEVSsimulator::computeTransition (stime_t t) {

// update the model time
model->setTime (t);

// if t = tN and input was received during the route portion of the
// coordinator's computeIO phase, trigger the confluent transition function
if (t == model->timeNext () && recvd_input) {
   model->delta_conf ();
   model->setTimeLast (t);
   }

// otherwise if t = tN but no input, compute internal transition function
else if (t == model->timeNext ()) {
   model->delta_int ();
   model->setTimeLast (t);
   }

// othewise, must need the external transition function
else if (recvd_input) {
   model->delta_ext ();
   model->setTimeLast (t);
   }

// start of new sim cycle, so reset input flag
recvd_input = false;

} 

void DEVSsimulator::input (port_t inport, object* value) {

simulator::input (inport, value);
// set the input flag for computeTransition
recvd_input = true;

}

DEVSsimulator::~DEVSsimulator () { }
