/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <adevs/adevs.h>
using namespace adevs;

// If a thread is available, get a parallel coordinator from the pool
simulator* SimulatorPool::getParallelCoordinator () {

// Coordinator to return
pDEVScoordinator* coord = NULL;
// If a threaded coordinator is already available, get it 
if (pcoords.getSize() > 0) 
   coord = (pDEVScoordinator*)pcoords.remove (0);
// Otherwise, make a new coordinator 
else 
   coord = new pDEVScoordinator ();
// Return the result
return coord;

}

// Repeat of getParallelCoordinator for parallel simulators
simulator* SimulatorPool::getParallelSimulator () {

pDEVSsimulator* sim = NULL;
if (psims.getSize() > 0) 
   sim = (pDEVSsimulator*)psims.remove (0);
else 
   sim = new pDEVSsimulator ();
return sim;

}

simulator* SimulatorPool::getSequentialSimulator () {

DEVSsimulator* sim = NULL;
// get a premade simulator if possible
if (sims.getSize () > 0) 
   sim = (DEVSsimulator*)sims.remove (0);
// otherwise allocate a new one
else 
   sim = new DEVSsimulator ();
return sim;
 
}

simulator* SimulatorPool::getSequentialCoordinator () {

DEVScoordinator* coord = NULL;
if (coords.getSize () > 0) 
   coord = (DEVScoordinator*)coords.remove (0);
else 
   coord = new DEVScoordinator ();
return coord;

}

simulator* SimulatorPool::wrap (devs* d) {

if (d->getType () == devs::ATOMIC) {
	DEVSsimulator* sim = NULL;
   if (d->getThreaded()) sim = (DEVSsimulator*)getParallelSimulator ();
   else sim = (DEVSsimulator*)getSequentialSimulator ();
   sim->setModel ((atomic*)d);
   dsmap.insert (d, sim);
   return sim;
   }
// otherwise, attach the network model to a DEVScoordinator
else {
	DEVScoordinator* coord = NULL;
   if (d->getThreaded()) coord = (DEVScoordinator*)getParallelCoordinator ();
   else coord = (DEVScoordinator*)getSequentialCoordinator ();
   coord->setModel ((devsn*)d);
   dsmap.insert (d, coord);
   return coord;
   }
      
return NULL;
}
     
void SimulatorPool::unwrap (simulator* s) {

// remove the (model, simulator) pair from the model->sim map
dsmap.remove (*s->getModel ());

// return the simulator to the appropriate pool
if (s->getType () == simulator::PSIM)
   psims.add(s);
else if (s->getType () == simulator::SIM)
   sims.add (s);
else if (s->getType () == simulator::PCOORD)
   pcoords.add(s);
else 
   coords.add (s);

}

SimulatorPool::~SimulatorPool () {

psims.empty ();
pcoords.empty ();
sims.empty ();
coords.empty ();

}
