/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000, 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <adevs/adevs.h>
using namespace adevs;

indexed::indexed():
object() {
	this->capacity = DEFAULT_CONTAINER_CAPACITY; 
	this->grow = DEFAULT_CONTAINER_GROWTH; 
	size = 0;
	table = new object* [capacity];
} // end of base constructor

indexed::indexed(unsigned int capacity, unsigned int grow):
object() {
	this->capacity = capacity;
	this->grow = grow;
	size = 0;
	table = new object* [capacity];
} // end of contructor

indexed::indexed(const indexed &src):
object() {
	capacity = src.capacity;
	grow = src.grow;
	size = src.size;
	table = new object* [capacity];
	for (unsigned int i = 0; i < size; i++)
		table[i] = src.table[i];
} // end of copy constructor

void indexed::swap(unsigned int i, unsigned int j) {
	object* temp = table[i];
	table[i] = table[j];
	table[j] = temp;
} // end of swap

void indexed::trim() {
	object** rtable = new object* [size];
	for (unsigned i = 0; i < size; i++) 
		rtable[i] = table[i];
	capacity = size;
	delete [] table;
	table = rtable;
} // end of trim

void indexed::enlarge(unsigned int adjustment) {
	capacity += adjustment;
	object** rtable = new object* [capacity];
	for (unsigned i = 0; i < size; i++) 
		rtable[i] = table[i];
	delete [] table;
	table = rtable;
} // end of enlarge

void indexed::append(const indexed& src) {
	int grow = src.getSize() - capacity - size;
	if (grow > 0)
		enlarge ((unsigned)grow);
	for (unsigned i = 0; i < src.getSize (); i++) 
		add(src[i]);
} // end of append

void indexed::empty() {
	for (unsigned i = 0; i < size; i++) {
		if (!duptree.contains((void*)table[i])) {
			duptree.insert((void*)table[i]);
			delete table[i];
			}
		}
	size = 0;
	duptree.clear_tree();
} // end of empty

void indexed::add(object *obj) {
	if (capacity == size) 
		enlarge(grow);
	table[size] = obj;
	size++;
} // end of add

object* indexed::remove(unsigned index) {
	object *last = table[index];
	size--;
	table[index] = table[size];
	return last;
} // end of remove

void indexed::removeAll() {
	size = 0;
}

indexed::~indexed() {
	delete [] table;
} // end of class indexed
