/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**

Model of a process in a distributed computation.  The process implements an
algorithm to control the execution of N programs that are trying to
operate on a shared state.  Each program has a copy of the state and
can request to operate on the state at any time.  Goal is to be sure that
every program sees the same operations in the same order.

*/

class process: public adevs::atomic {

public:
// creates a process with a process ID (should be the same ID given to the attached
// program) and a priori knowledge about the number of processes in the system.
	process (unsigned pid, unsigned num_processes);

	void reset ();
	void delta_int ();
	void delta_ext ();
	void delta_conf ();
	void output_func ();

	~process ();

// input ports.  All of these expect message objects as values.
	static const adevs::port_t recv;
	static const adevs::port_t prog_in;

// output ports.  The send port produces messages, the prog_out port produces a queue
// of messages.
	static const adevs::port_t send; 
	static const adevs::port_t prog_out;

private:

// state
	unsigned pid, c, gc, num_procs;
	bool* sync;
	adevs::queue** waiting; 
	adevs::indexed safe;

// I/O buffers
	adevs::indexed pout, dout;
	
	void execute ();
	void _recv (const message& msg);
	void _send (adevs::object* obj, adevs::port_t port);

};
