#include <adevs/adevs.h>
#include <stdlib.h>
#include <iostream.h>
#include "systolicArray.h"
#include "host.h"
using namespace adevs;

/**
Experimental frame for the systolic array.
*/
class ef: public staticDigraph {

public:	
// constructor takes an array of weights, array of x values,
// and the lengths of both arrays.
		ef (const double* weights, const double* x, 
			unsigned w, unsigned coeffs);
		~ef ();

};

/**
Constructor.  Creates a host and systolic array then establishes coupling
between them.
*/
ef::ef (const double* weights, const double* x, unsigned w, unsigned coeffs):
staticDigraph () {

// create array
systolicArray* array = new systolicArray (weights, w);
// create host
host* user = new host (x, coeffs, w);

// add array and host to the digraph model
add (user);
add (array);

// establish coupling
couple (user, user->xout, array, array->xin);
couple (user, user->yout, array, array->yin);
couple (array, array->yout, user, user->yin);

}

ef::~ef () { }


/**
Main program
**/

int main () {

double* x;
double* w;
unsigned wc, xc;
unsigned threads;

// get number of weights and coefficients (x's) from stdin
std::cout << "# Weights: ";
std::cin >> wc;
std::cout << "# Coefficients: ";  
std::cin >> xc;

// check for allowable number of w's and x's
if (xc + 1 - wc <= 0) {
   std::cout << "n + 1 - k <= 0" << std::endl;
   exit (-1);
   }

x = (double*) malloc (sizeof (double) * xc);
w = (double*) malloc (sizeof (double) * wc);

// get values for weights
std::cout << "Weights: ";
for (unsigned i = 0; i < wc; i++) 
   std::cin >> w[i];

// get x values
std::cout << "Coefficients: ";
for (unsigned i = 0; i < xc; i++) 
   std::cin >> x[i];

std::cout << std::endl << "n = " << xc << ", k = " << wc << std::endl;
// Get the number of threads for execution
std::cout << "# Threads: " << std::endl;
std::cin >> threads;

try {
   devssim::setThreadLimit (threads);
   // create the simulator
   devssim sim (new ef (w, x, wc, xc));
   // run the simulator.
   sim.run ();
   } 

catch (DEVSException& err) {
   cout << err.getError () << endl;
   }

// clean up and exit
free (x);
free (w);

}
