#include <adevs/adevs.h>
#include <stdlib.h>
#include <iostream.h>
#include "systolicArray.h"
#include "host.h"
using namespace adevs;

/**
Systolic array array model that has its I/O processed via the
step and computeNextOutput functions.
*/

/**
Main program
**/
int main () {

double* x;
double* w;
unsigned wc, xc;
unsigned threads;

// get number of weights and coefficients (x's) from stdin
std::cout << "# Weights: ";
std::cin >> wc;
std::cout << "# Coefficients: ";  
std::cin >> xc;

// check for allowable number of w's and x's
if (xc + 1 - wc <= 0) {
   cout << "n + 1 - k <= 0" << endl;
   exit (-1);
   }

x = (double*) malloc (sizeof (double) * xc);
w = (double*) malloc (sizeof (double) * wc);

// get values for weights
std::cout << "Weights: ";
for (unsigned i = 0; i < wc; i++) 
   std::cin >> w[i];

// get x values
std::cout << "Coefficients: ";
for (unsigned i = 0; i < xc; i++) 
   std::cin >> x[i];

std::cout << std::endl << "n = " << xc << ", k = " << wc << std::endl;
std::cout << "# Threads: " << std::endl;
std::cin >> threads;

try {

   // create array
   systolicArray* array = new systolicArray (w, wc);
   // create the simulator
   devssim::setThreadLimit (threads);
   devssim sim (array);
   // initialize the simulator
   sim.reset ();

   // run the simulator in a stepwise fashion
   // deliver one value per unit of time
   indexed input, output;
   for (unsigned count = 0; count < xc || sim.timeNext() < INFINITY; count++) {

      // create the input for the next cycle
      int y = count + 1 - wc;
      if (count < xc) 
         input.add (new port_value (array->xin, new dbl (x[count])));
      if (y >= 0 && count < xc)
         input.add (new port_value (array->yin, new dbl (0.0)));

      // get the output and report it
      sim.computeNextOutput (output);
      for (unsigned i = 0; i < output.getSize(); i++) {
         port_value* p = (port_value*)output[i];
         dbl* result = (dbl*)p->value;
         if (p->port == array->yout) {
            std::cout << "y = " << result->value << " @ t = ";
            std::cout << sim.timeNext () << std::endl;
            }
         }
      output.empty ();

      // compute the next transition 
      sim.step (1.0, input);
      }
      
   } 

catch (DEVSException& err) {
   cout << err.getError () << endl;
   }

// clean up and exit
free (x);
free (w);

}
