
/**
Job generator.  Produces jobs with a uniformly distributed job size
and with uniformly distribted inter-job time.
*/

#ifndef _GENR_
#define _GENR_
#include "types.h"

class genr: public adevs::atomic {

public:		genr (adevs::stime_t period, adevs::stime_t job_size,
			int max_job_count = INT_MAX);

		void reset ();
		void delta_int ();
		void delta_ext ();
		void delta_conf ();
		void output_func ();

		~genr ();

/// Genr stops producing jobs when it receives a stop message
		static const adevs::port_t stop;
/// New jobs appear on the out port
		static const adevs::port_t out;

private:	
/// Mean job size and period
		adevs::stime_t period, job_size;
/// Number of jobs so far
		int count;
/// Size of the next outgoing job
		double next_job_size;
/// Number of jobs to produce
		int max_job_count;
/// Random variable
		adevs::rv r;

};

#endif
