#include <time.h>
#include <list>
#include <adevs/adevs.h>
#include "proc.h"
using namespace adevs;

const port_t proc::in = 0;
const port_t proc::out = 1;

// conservative lookahead function
stime_t proc::lookahead () const {

stime_t min = INFINITY;

std::list<job>::const_iterator i;
for (i = runq.begin(); i != runq.end() && !done; i++) 
   if (min > ((*i).t - time_slice)) min = (*i).t - time_slice;   

if (min == INFINITY || min < 0.0)
   return 0.0;

return min;

}

proc::proc (int pid, stime_t time_slice):
atomic ("proc") {

addInputPort (in);
addOutputPort (out);

this->time_slice = time_slice;
this->pid = pid;
done = false;

}

void proc::reset () {

done = false;
runq.erase (runq.begin(), runq.end());
passivate ();

}

void proc::delta_int () {

done = false;
if (runq.size () == 0) {
   passivate ();
   return;
   }

j = runq.front ();
runq.pop_front ();
if (j.t <= time_slice) {
   hold (j.t);
   j.t = 0.0;
   done = true;
   }
else {
   hold (time_slice);
   j.t -= time_slice;
   runq.push_back (j);
   }

}

void proc::delta_ext () {

bag* values = inputOnPort (in);
for (unsigned i = 0; i < values->getSize(); i++) {
   job* _j = (job*)values->getObj (i);
   if (_j->pid == pid) {
      runq.push_back (*_j);
      }
   }

if (timeNext() == INFINITY && runq.size() > 0)
   hold (0);

}
 
void proc::delta_conf () {

delta_int ();
delta_ext ();

}

void proc::output_func () {

if (done)
   output (out, new job (j));

}

proc::~proc () {

runq.erase (runq.begin(), runq.end());

}

