#include <iostream>
#include <map>
#include <adevs/adevs.h>
#include "transd.h"
using namespace adevs;

const port_t transd::stop = 0;
const port_t transd::ariv = 1;
const port_t transd::solv = 2;

transd::transd (stime_t obs_time):
atomic ("transd") {

setThreaded(false);
addInputPort (ariv);
addInputPort (solv);
addOutputPort (stop);

this->obs_time = obs_time;

}

void transd::reset () {

solved.clear ();
arrived.clear ();
hold (obs_time);

}

void transd::delta_int () {

report_results ();
passivate ();

}

void transd::delta_ext () {

if (timeNext() == INFINITY) return;

bag* values = inputOnPort (ariv);
for (unsigned i = 0; i < values->getSize(); i++) {
   job* j = (job*)values->getObj (i);
   arrived[*j] = timeCurrent ();
   }

values = inputOnPort (solv);
for (unsigned i = 0; i < values->getSize(); i++) {
   job* j = (job*)values->getObj (i);
   solved[*j] = timeCurrent ();
   }

}

void transd::delta_conf () {

delta_int (); 
delta_ext ();

}

void transd::output_func () {

output (stop, new object ());

}

void transd::report_results () {

std::cout << std::endl;
std::cout << "Obs time: " << timeCurrent() << std::endl;
std::cout << "Arrived: " << arrived.size() << std::endl;
std::cout << "Solved: " << solved.size() << std::endl;

if (solved.size () == 0 || timeCurrent () == 0.0) return;

stime_t avg_ta = 0.0;
stime_t thruput = 0.0;

for (jobmap::iterator i = solved.begin(); i != solved.end(); i++) {
   stime_t end = (*i).second;
   stime_t start = arrived[(*i).first];
   avg_ta += (end - start);
   }

avg_ta /= (double)solved.size();
thruput = (double)solved.size() / timeCurrent();

std::cout << "Avg TA: " << avg_ta << std::endl;
std::cout << "Throughput: " << thruput << std::endl;

}
