/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef _VECTOR_
#define _VECTOR_

/**

A 3D vector

*/
#include <adevs/adevs.h>
#include <math.h>

class vector: public adevs::object {

public:
// Zero vector
	static const vector ZERO;

// Vector components
	double x, y, z;

// constructors
	inline vector ():adevs::object() { x = y = z = 0.0; }
	inline vector (double _x, double _y, double _z ):
	adevs::object() { x = _x; y = _y; z = _z; }
	inline vector (const double* data):adevs::object() {
	   x = data[0]; y = data[1]; z = data[2];
	   }
	inline vector (const vector& src):
	adevs::object(src) { x = src.x; y = src.y; z = src.z; }
// returns the length of the vector
	inline double length () const {
	   return sqrt (x*x + y*y + z*z);
	   }

// binary operators 	
	inline vector operator+ (const vector& v) const {
		return vector (v.x+x, v.y+y, v.z+z); } 
	inline vector operator- (const vector& v) const {
		return vector (x-v.x, y-v.y, z-v.z); } 
	inline vector operator* (double _x) const {
		return vector (x*_x, y*_x, z*_x); } 
	inline vector operator/ (double _x) const {
		return vector (x/_x, y/_x, z/_x); } 

// assignment operators
	inline vector& operator+= (const vector& v) {
		x += v.x; y += v.y; z += v.z; return *this; }
	inline vector& operator-= (const vector& v) {
		x -= v.x; y -= v.y; z -= v.z; return *this; }
	inline vector& operator*= (double _x) {
		x *= _x; y *= _x; z *= _x; return *this; }
	inline vector& operator/= (double _x) {
		x /= _x; y /= _x; z /= _x; return *this; }
	inline vector& operator= (const vector& v) {
		x = v.x; y = v.y; z = v.z; return *this; }

// equivalence testing
	inline bool operator== (const vector& v) const {
		return (x == v.x && y == v.y && z == v.z); } 
	inline bool operator!= (const vector& v) const {
		return (x != v.x && y != v.y && z != v.z); } 

	~vector () { }

	adevs::object* clone () const { return new vector (*this); }

};

inline vector floor (const vector& v) {
	return vector (floor(v.x),floor(v.y),floor(v.z)); }
inline vector operator* (double x, const vector& v) {
	return vector (v.x*x, v.y*x, v.z*x); }

#endif
