/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/

/**

The game display and controller constructed as an atomic model.

*/

class viewer: public adevs::atomic {

// utility class to hold position histories
friend class __pos;

public:

// input ports
	static const adevs::port_t ball_motion;
	static const adevs::port_t block_position;
	static const adevs::port_t paddle_motion;
	static const adevs::port_t ball_status;

// output ports
	static const adevs::port_t paddle_control;
	static const adevs::port_t stop; 

// Creates a view window.  Calls exit() on failure.
	viewer (char** argv, int argc, int width, int height);

	void reset ();
	void delta_int ();
	void delta_ext ();
	void delta_conf ();
	void output_func ();

	inline bool hasQuit () const { return quit; }
	inline int getScore () const { return score; }

// Closes the window.  Frees all display resources.
	~viewer ();
private:
	static const adevs::stime_t PERIOD;
// Draws a circle centered on (x, y).
	void drawBall (double x, double y, double radius, __pos* old);
// Draws a rectangle centered at (x, y) with appropriate dimensions.
	void drawBlock (double x, double y, double width, double height, eStatus status);
// Draws a rectangle centered at (x, y) with appropriate dimensions.
	void drawPaddle (double x, double y, double width, double height, __pos* old);
// Draws the score string in the upper left corner.
	void drawScore (int score, bool first = false);
// Draws the balls remaining string  in the upper left corner.
	void drawBallCount (int balls, bool first = false);
// Check the window event queue
	void checkEvents ();

	Display* display;
	int screen;
	Window win;
	GC gc;

	XSizeHints size_hints;
	XEvent event;
	long event_mask;

	adevs::hashtable pos_history;

	paddleControlEvent* control_event;

	int score;
	char last_score [20];
	char last_ball_count [20];
	int balls;
	bool quit;
	int block_count;

};
