#include <adevs/adevs.h>
#include <iostream>
#include <assert.h>
#include "genr.h"
/**

Simple generator test.  Targets the step (stime_t, indexed&)
method.  Test should throw an exception due to the
time of next event being after the step time.

**/

const port_t genr::out = 0;
const port_t genr::in1 = 1;
const port_t genr::in2 = 2;

genr::genr (stime_t period, int ticks):atomic () {

this->period = period;
this->tick_limit = ticks;
this->tick_count = 0;

addInputPort (in1);
addInputPort (in2);
addOutputPort (out);

}

void genr::reset () {

tick_count = 0;
hold (period);

}

void genr::delta_ext () { 

intg sum = 0;

bag* values = inputOnPort (in1);
for (unsigned i = 0; i < values->getSize (); i++) {
   intg* j = (intg*)values->getObj (i);
   sum += *j;
   }

values = inputOnPort (in2);
for (unsigned i = 0; i < values->getSize (); i++) {
   intg* j = (intg*)values->getObj (i);
   sum += *j;
   }

std::cout << "Sum is " << sum.value << std::endl;
assert (sum == (1+2+3+4));
std::cout << "Got expected sum! @ t = " << timeCurrent () << std::endl; 
passivate (); 

}

void genr::delta_int () { 

std::cout << "tick_count = " << tick_count << " @ t = " << timeCurrent () << std::endl;
tick_count++;

if (tick_count < tick_limit) {
   hold (period);
   std::cout << "period = " << period << std::endl;
   std::cout << "tN = " << timeNext () << std::endl;
   std::cout << "tL = " << timeLast () << std::endl;
   } 

else
   passivate ();

} 

void genr::delta_conf () { assert (false); }

void genr::output_func () {

output (out, new intg (tick_count));

}

genr::~genr () { }

int main () {

genr* g = new genr (1.0, 10);
devssim sim (g);
sim.reset ();

indexed input;
for (int i = 1; i <= 2; i++) 
   input.add (new port_value (genr::in1, new intg (i)));
for (int i = 3; i <= 4; i++) 
   input.add (new port_value (genr::in2, new intg (i)));

try {

   sim.step (1.0);
   sim.step (1.5, input);

   }
catch (ScheduleException &err) {
   std::cout << "Caught exception!" << std::endl;
   std::cout << err.getError () << std::endl;
   exit (0);
   }

assert (false);

}
