#include <adevs/adevs.h>
#include <iostream>
#include <assert.h>
#include "genr.h"

/**

Simple generator test.  Targets the computeNextOutput (indexed&)
method.  

**/

// special simulator for test
class _devssim: public devssim {

public:
	_devssim (atomic* model);
	virtual bool halt ();

private:
	int count;

};

_devssim::_devssim (atomic* model):devssim(model) { count = 0; }

bool _devssim::halt () {

indexed output;
computeNextOutput (output);
assert (output.getSize () == 1);
port_value* pv = (port_value*)output[0];
intg* i = (intg*)pv->value;
assert (i->value == count);
count++;

return devssim::halt ();

}

// start the test genr model

const port_t genr::out = 0;
const port_t genr::in1 = 1;
const port_t genr::in2 = 2;

genr::genr (stime_t period, int ticks):atomic () {

this->period = period;
this->tick_limit = ticks;
this->tick_count = 0;

addInputPort (in1);
addInputPort (in2);
addOutputPort (out);

}

void genr::reset () {

tick_count = 0;
hold (period);

}

void genr::delta_ext () { 

assert (false);

}

void genr::delta_int () { 

std::cout << "tick_count = " << tick_count << " @ t = " << timeCurrent () << std::endl;
tick_count++;

if (tick_count < tick_limit) {
   hold (period);
   std::cout << "period = " << period << std::endl;
   std::cout << "tN = " << timeNext () << std::endl;
   std::cout << "tL = " << timeLast () << std::endl;
   } 

else
   passivate ();

} 

void genr::delta_conf () { assert (false); }

void genr::output_func () {

output (out, new intg (tick_count));

}

genr::~genr () { }

int main () {

genr* g = new genr (1.0, 10);
_devssim sim (g);
sim.reset ();

sim.run ();

std::cout << "Test done!" << std::endl;

}
